/*! Build-0.2: colorpicker.core.js colorpicker.hsv-square.js colorpicker.info.js colorpicker.palette.js colorpicker.history.js colorpicker.related.js */
/*!
 * jQuery Colour Picker 0.2
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT and GPL licenses.
 */
/* Author: Mark Gibson (jollytoad at gmail dot com)
 *
 * A colour picker widget framework using jQuery Roles & jQuery Colour library.
 *
 * Depends:
 *   roles.core.js
 *   color.object.js
 */
(function($) {

$('.colorpicker')
	.roleStage('bind', function() {
		$(this)
			// ---- Dialog button actions ----
			
			.roleAction('action-apply', function(event) {
//				console.log('apply');
				var color = $.data(this, 'color').toString();
				$(this).closest('[data-color]').attr('data-color', color);
				return false;
			})
			
			.roleAction('action-ok', function(event) {
//				console.log('ok');
				$(this)
					.trigger('action-apply')
					.trigger('action-cancel');
				return false;
			})
			
			.roleAction('action-cancel', function(event) {
//				console.log('cancel');
				$(this)
					.closest(':role(dialog)')
					.attr('aria-hidden', true);
				return false;
			})

			// ---- Color modification events ----

			.bind('color.colorpicker', function(event, fn) {
				var color;
				
				if ( $.isFunction(fn) ) {
					var args = Array.prototype.slice.call(arguments, 2);
					color = fn.apply($.data(this, 'color'), args);
				} else if ( $.Color.isInstance(fn) ) {
					color = fn;
				} else if ( fn ) {
					color = $.Color(fn);
				}
				
				if ( color ) {
					$.data(this, 'color', color);
				}
			});
	})
	.roleStage('activate', function() {
		var color = $(this).closest('[data-color]').attr('data-color') || 'transparent';
		$(this).trigger('color', [color]);
	});

})(jQuery);

/*
 * jQuery Colour Picker - HSV widget 0.2
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT and GPL licenses.
 */
/* Author: Mark Gibson (jollytoad at gmail dot com)
 *
 * Provides a Saturation/Value area widget, and a Hue slider.
 *
 * Depends:
 *   colorpicker.core.js
 *   roles.core.js
 *   ui.surface.js
 *   ui.area.js
 *   ui.vertical.js
 *   mutations.core.js
 *   mutations.data.js
 *   color.object.js
 *   color.rgb.js
 *   color.hsv.js
 */
(function($) {

$('.colorpicker .satval-area')
	.roleStage('bind', function() {
		var square = $(this);
		square
			.area()
			.bind('pointdrag.colorpicker', function(event, ui) {
				$(event.target).trigger('color', [$.Color.fn.modify, $.Color([null, ui.x, 1 - ui.y], 'HSV')]);
			})
			
			.closest('.colorpicker')
				.bind('data.@color.colorpicker', function(event) {
					var color = event.newValue;
					$.roles.whenVisible(square, 'colorpicker-satval-area', function() {
						var hsv = color.toHSV();
						square
							.css('background-color', $.Color([hsv[0],1,1], 'HSV').toCSS())
							.area('option', {x: hsv[1], y: 1 - hsv[2] });
					});
				})
			.end();
	});

$('.colorpicker .hue-slider')
	.roleStage('bind', function() {
		var slider = $(this);
		slider
			.vertical()
			.bind('pointdrag.colorpicker', function(event, ui) {
				$(event.target).trigger('color', [$.Color.fn.modify, $.Color([ui.value, null, null], 'HSV')]);
			})
			
			.closest('.colorpicker')
				.bind('data.@color.colorpicker', function(event) {
					var color = event.newValue;
					$.roles.whenVisible(slider, 'colorpicker-hue-slider', function() {
						slider.vertical('option', { value: color.toHSV()[0] });
					});
				})
			.end();
	});

})(jQuery);

/*
 * jQuery Colour Picker - Colour info widgets 0.2
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT and GPL licenses.
 */
/* Author: Mark Gibson (jollytoad at gmail dot com)
 *
 * Provides a colour sample, CSS/HEX input, and separate channel inputs.
 *
 * Depends:
 *   colorpicker.core.js
 *   roles.core.js
 *   mutations.core.js
 *   mutations.data.js
 *   color.object.js
 *   color.rgb.js
 *   color.parse.js
 *   color.<space>.js - any other required colour spaces
 */
(function($) {

$('.colorpicker .color-sample')
	.roleStage('bind', function() {
		var sample = $(this);
		sample
			.closest('.colorpicker')
				.bind('data.@color.colorpicker', function(event) {
					sample
						.css('background-color', event.newValue.toCSS())
						.attr('title', event.newValue.name || event.newValue.toHEX());
				})
			.end();
	});

$('.colorpicker input.color-string')
	.roleStage('bind', function() {
		var string = $(this);
		string
			.bind('change.colorpicker', function() {
				$(this).trigger('color', [this.value]);
			})
			.closest('.colorpicker')
				.bind('data.@color.colorpicker', function(event) {
					string.val(event.newValue.toHEX());
				})
			.end();
	});

$('.colorpicker .channel-inputs input.channel')
	.roleStage('bind', function() {
		var input = $(this);
		input
			.bind('change.colorpicker', function() {
				var space = $(input).closest('[data-color-space]').attr('data-color-space'),
					index = input.attr('data-channel-index'),
					factor = input.attr('data-channel-factor') || 1,
					tuple = [null,null,null];
				tuple[index] = parseInt(input.val(), 10) / factor;
				input.trigger('color', [$.Color.fn.modify, new $.Color(tuple, space)]);
			})
			.closest('.colorpicker')
				.bind('data.@color.colorpicker', function(event) {
					var space = $(input).closest('[data-color-space]').attr('data-color-space'),
						index = input.attr('data-channel-index'),
						factor = input.attr('data-channel-factor') || 1;
					input.val(Math.round(event.newValue.to(space)[index] * factor));
				})
			.end();
	});

})(jQuery);

/*
 * jQuery Colour Picker - Colour info widgets 0.2
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT and GPL licenses.
 */
/* Author: Mark Gibson (jollytoad at gmail dot com)
 *
 * Provides a colour sample, CSS/HEX input, and separate channel inputs.
 *
 * Depends:
 *   colorpicker.core.js
 *   roles.core.js
 *   roles.aria.js
 *   color.object.js
 *   color.palette.*.js - optional pre-defined palettes
 */
(function($) {

function createSwatch(color, key) {
	return $('<div></div>')
		.attr({
			'role': 'option',
			'title': (color.name ? color.name + ': ' : '') + color.toString()
		})
		.data('color', color)
		.data('key', key)
		.css('background-color', color.toCSS());
}

$('.colorpicker :role(listbox)[data-palette]')

	.roleStage('bind', function() {
		$(this)
		
			// ---- Custom attributes ----
			
			.bind('attr.@data-palette.colorpicker', function(event) {
				var self = this,
					palette = event.newValue;
				
				function refresh() {
					$.roles.whenVisible(self, 'color-palette-'+palette, function() {
						$(self).empty();
				
						$.each($.color.palette[palette] || {}, function(i, c) {
							var color = new $.Color(c);
		
							if (!color.name && typeof i === 'string') {
								color.name = i;
							}
							
							createSwatch(color, i).appendTo(self);
						});
						
						$(self).roleSetup();
					});
				}
				
				if ( event.prevValue ) {
					$.roles.whenVisible(self, 'color-palette-'+event.prevValue);
					$(document).unbind('color-palette-'+event.prevValue);
				}
				
				if ( palette ) {
					$(document).bind('color-palette-'+palette, refresh);
					refresh();
				}
			})
		
			// ---- Actions ----

			// Focus swatch in previous row
			.roleAction('action-prev-row', function(event) {
				var left = event.target.offsetLeft;
				$(event.target).prevAll().filter(function() { return this.offsetLeft === left; }).eq(0).focus();
				return false;
			})
			
			// Focus swatch in next row
			.roleAction('action-next-row', function(event) {
				var left = event.target.offsetLeft;
				$(event.target).nextAll().filter(function() { return this.offsetLeft === left; }).eq(0).focus();
				return false;
			})
			
			// Choose the colour swatch
			.roleAction('action-select', function(event) {
				var color = $(event.target).closest(':role(option)').data('color');
				if ( color ) {
					$(event.target).trigger('color', [color]);
				}
				return false;
			})
			
			// ---- Mouse ----
			
			.roleBind('click', 'action-select')
			
			// ---- Keys ----
			
			.roleKey('up', 'action-prev-row')
			.roleKey('down', 'action-next-row')
			.roleKey('enter', 'action-select')

			.end();
	})
	
	.roleStage('init', function() {
		$(this)
			.initMutation('attr', 'data-palette');
	});

})(jQuery);

/*
 * jQuery Colour Picker - Colour history palette 0.2
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT and GPL licenses.
 */
/* Author: Mark Gibson (jollytoad at gmail dot com)
 *
 * Record colour selection history in a palette
 *
 * Depends:
 *   roles.core.js
 *   mutations.core.js
 *   mutations.data.js
 *   color.core.js
 *   jquery.defer.js - optional but recommended if used with widgets that generate
 *                     a lot of color data mutation events, eg. hsv-square
 */
(function($) {

var opts = {
	palette: 'history',
	defer: 1000
};

function colorChange(event) {
	var key = event.newValue.toString(),
		palette = $.color.palette[opts.palette];
	
	if ( !palette ) {
		palette = $.color.palette[opts.palette] = {};
	}
	
	if ( !palette[key] ) {
		palette[key] = event.newValue;
		$(event.target).trigger(
			$.mutations.event('color-palette-'+opts.palette,
					{ attrName: key, newValue: event.newValue })
		);
	}
}

$('.colorpicker')
	.roleStage('bind', function() {
		$(this)
			// ---- Data mutations ----
			
			.bind('data.@color.colorpicker',
				$.defer && opts.defer ? $.defer(opts.defer, colorChange) : colorChange)
			
			// ---- Actions ----
			
			.roleAction('action-history-clear', function() {
				$.color.palette[opts.palette] = {};
				$(this).trigger('color-palette-'+opts.palette);
				return false;
			});
	});

})(jQuery);

/*
 * jQuery Colour Picker - Related colours palette 0.2
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT and GPL licenses.
 */
/* Author: Mark Gibson (jollytoad at gmail dot com)
 *
 * Generate a palette of colours related to the current colour.
 *
 * Depends:
 *   roles.core.js
 *   mutations.core.js
 *   mutations.data.js
 *   color.core.js
 *   color.related.js
 *   jquery.defer.js - optional but recommended if used with widgets that generate
 *                     a lot of color data mutation events, eg. hsv-square
 */
(function($) {

var opts = {
	palette: 'related',
	defer: 1000
};

function colorChange(event) {
	$.color.palette[opts.palette] = $.Color(event.newValue).related();	
	$(event.target).trigger('color-palette-'+opts.palette);
}

$('.colorpicker')
	.roleStage('bind', function() {
		$(this)
			// ---- Data mutations ----
			
			.bind('data.@color.colorpicker',
				$.defer && opts.defer ? $.defer(opts.defer, colorChange) : colorChange);
	});

})(jQuery);

