/*jslint regexp:false */
/*global jQuery, window, document */

(function($) {

	var urlPrefix = $('#confluence-context-path').attr('content') + '/plugins/servlet/builder/resource/',
		panels,
		colorProps = {
			'backgroundColor': 'Background Colour',
			'borderColor': 'Border Colour',
			'color': 'Text Colour'
		},
		popupTrigger;
	
	$.color.palette.builder = {};
	$.color.palette.confluence = {};

	function revert(input, origin) {
		input.trigger('atb-field-revert', origin);
	}

	var actions = {
	
		color: function(input) {
			var color = input.val(),
				palette = $('#layout-manager').data('atb-palette') || {};
			
			color = palette[color] || color;
			
			// Convert 'transparent' to 'white' until colour picker supports alpha values
			if ( color === 'transparent' || !color ) {
				color = 'white';
			}
						
			$('#layout-manager-colorpicker')
				.unbind('.css-color')
				.attr('data-color', color)
				.bind('attr.@data-color.css-color', function(event) {
					input.val(event.newValue).trigger('change');
				})
				.trigger('atb-activate')
				.find('.colorpicker')
					.trigger('color', [color]);
		},
		
		parent: revert,
		overlay: revert,
		
		value: function(input, action, value) {
			input.val(value).trigger('change');
		},
		
		none: function() {}
	};
	
	var init = {
		// Build a menu of images from the layout resources
		'css-popup-background-image': function() {
			
			var imageMenu = $('#css-popup-images'),
				layoutData = $('#layout-manager').data('atb-layout'),
				valuePrefix = layoutData.layoutId + '::',
				thumbPrefix = urlPrefix + layoutData.layoutId + '/',
				thumbCount = 0;

			// TODO: rebuild menu only when the resources list has changed
			
			// Remove image resources
			$('li.css-thumb', imageMenu).remove();
			
			$.each(layoutData.resources, function(key, data) {
				if ( /^image/.test(data.contentType) ) {
					$('<li class="css-thumb"/>')
						.attr('title', key)
						.append($('<a href="#" class="css-menu-item"/>')
							.attr('data-value', valuePrefix + key)
							.append($('<img/>')
								.attr('src', thumbPrefix + key)
								.attr('alt', key)
							)
						)
						.prependTo(imageMenu);
					
					thumbCount += 1;
				}
			});
			
			$('li.css-resources-none', imageMenu).toggleClass('hidden', !!thumbCount);
		},
		
		// Build colour palette menus
		'css-popup-color': function() {
			var layoutData = $('#layout-manager').data('atb-layout'),
				builderMenu = $('#css-popup-colors-builder').empty(),
				palette = $('#layout-manager').data('atb-palette') || {},
				builderPalette = $.color.palette.builder,
				confluencePalette = $.color.palette.confluence;
			
			// Build list of panel names
			if ( !panels ) {
				panels = {};
				$('#layout-manager .layout-preview [data-panel][title]').each(function() {
					var that = $(this);
					panels[that.attr('data-panel')] = that.attr('title');
				});
			}
		
			// Builder palette
			$.each(panels, function(panelId, panelName) {
				var panelData = layoutData[panelId];
				$.each(colorProps, function(propId, propName) {
					$.each((panelData[propId] || '').split(/\s+/), function(i, value) {					
						if ( value ) {
							var color = $.Color(palette[value] || value).setName(panelName + ', ' + propName);
							
							$('<li class="css-color"/>')
								.append($('<a href="#" class="css-menu-item">&nbsp;</a>')
									.attr('data-value', value)
									.attr('title', color.name + ': ' + color.toString())
									.css('background-color', color.toCSS())
								)
								.appendTo(builderMenu);
							
							builderPalette[panelId+'/'+propId+'/'+i] = color;
						}
					});
				});
			});
			
			// Confluence palette
			$('#css-popup-colors-confluence a').each(function() {
				var that = $(this),
					id = that.attr('data-value'),
					title = that.attr('title').replace(/:.*$/, ''),
					color = palette[id];
				
				that.attr('title', title + (color ? ': ' + color : ''))
					.css('background-color', color || 'transparent');
				
				if ( color ) {
					confluencePalette[id] = $.Color(color).setName(title);
				}
			});
		}
	};
	
	function popup(menu, input, btn) {
		var id = menu.attr('id');
		
		// Perform menu initialization
		if ( init[id] ) {
			menu.each(init[id]);
		}
		
		menu.appendTo('body')
			.removeClass('hidden')
			.position({
				my: 'right center',
				at: 'left center',
				of: btn,
				offset: '-4 0',
				collision: 'none'
			})
			.bind('click.css-menu', function(event) {
				var a = $(event.target).closest('a'),
					value = a.attr('data-value'),
					action = a.attr('data-action') || (value === undefined || value === null ? 'none' : 'value');
				
				actions[action](input, action, value);
				
				// Prevent links from being followed
				event.preventDefault();
			});
		
		// Hide the menu when a click occurs anywhere in the document
		$(document).one('click.css-menu', function(event) {
				menu.addClass('hidden')
					.unbind('.css-menu');
				
				popupTrigger = undefined;
			});
	}

	// Wire-up the popup activation buttons
	$('#layout-manager-panel-editor .css-button')

		// Hide non functioning buttons
		.filter(function() {
			var popup = $.dt.attr(this, 'aria-controls', 'idrefs').length;
			$(this).toggleClass('hidden', !popup);
			return !!popup;
		})

		.bind('click.css-button', function(event) {
			if ( this !== popupTrigger ) {
				var input = $(event.target).closest('[data-ref]').find('.css-input'),
					menu = $.dt.attr(this, 'aria-controls', 'idrefs'),
					btn = this;
			
				// The timeout ensure that the current event queue is cleared.
				// This prevents the click to close popup handler from being fired.
				window.setTimeout(function() {
					popup(menu, input, btn);
					popupTrigger = btn;
				}, 0);
			}
			event.preventDefault();
		});

})(jQuery);

