/*global jQuery, ManageLayoutsDWR, alert, confirm, prompt */

// Layout Data Agent

// binds: atb-layout-save, atb-layout-reparent, atb-layout-clear, atb-layout-delete, atb-layout-new, atb-layout-duplicate
// triggers: atb-layout-select, atb-layout-reload, atb-hierarchy-load

// also see layouts/loader.js for the agent that fetches layout data

(function($) {

	function dataContext(event) {
		return $(event.target).closest('.atb-layout-data');
	}

	function cloneLayout( event, asChild ) {
		var newId, newLayout,
			context = dataContext(event),
			srcLayout = context.data('atb-layout'),
			hierarchy = context.data('atb-hierarchy');

		if ( !srcLayout.hasPermission('child') ) {
			alert("You do not have permission to create layouts based on the currently selected layout");
			return;
		}

		// Activate the overview tab  TODO: move this to layout-manager.js
//		$('#layout-manager-tablist').attr('aria-activedescendant', 'tab-description');

		function requestLayoutId() {
			var id = prompt("Enter Layout ID");
		
			if ( id !== null ) {
				if ( id.length < 1 ) {
					alert("Please enter a layout ID");
					return requestLayoutId();
				}
				id = id.toUpperCase();
				if ( !id.match(/^[A-Z0-9]+$/) ) {
					alert("Invalid layout ID : " + id);
					return requestLayoutId();
				}
				if ( hierarchy[id] ) {
		            alert("Layout ID '" + id + "' already in use, please select another!");
		            return requestLayoutId();
		        }
		    }
		    return id;
		}

		// Ask for a new Id for the layout
		newId = requestLayoutId();

		if ( !newId ) { return; }

		newLayout = $.overlay(srcLayout);
		
		if ( asChild ) {
			newLayout.parentId = srcLayout.layoutId;
			newLayout.importCSS = "/* empty */";
			newLayout.customCSS = "/* empty */";
			newLayout.ieCSS = "/* empty */";
		} else {
			// Remove resources from new layout
			// FIXME: Really need to duplicate resources in the backend too
			newLayout.resources = {};
		}
		
		newLayout.layoutId = newId;
		newLayout.name = (asChild ? "Child of " : "Duplicate of ") + srcLayout.layoutId;
		newLayout.description = newLayout.name + "\n" + srcLayout.description;

		if ( !hierarchy[newId] ) { hierarchy[newId]=[]; }

		ManageLayoutsDWR.setLayout(newLayout, function() {
			$(event.target)
				.trigger('atb-layout-select', [ newId ])
				.trigger('atb-hierarchy-load');
		});
	}

	$(document)
		// Save the layout data
		.bind('atb-layout-save', function(event) {
			ManageLayoutsDWR.setLayout(dataContext(event).data('atb-layout'), function() {
				$(event.target).trigger('atb-layout-reload', [ { keepDialog:true } ]);
			});
		})

		// Change the parent of a layout
		.bind('atb-layout-reparent', function(event, childId, parentId) {
			ManageLayoutsDWR.reparentLayout(childId, parentId, function(status) {
				if (status) { alert(status); }
				
				$(event.target)
					.trigger('atb-hierarchy-load')
					.trigger('atb-layout-reload');
			});
		})
		
		// Clear all customizations in the current layout
		.bind('atb-layout-clear', function(event) {
			var context = dataContext(event),
				layoutData = context.data('atb-layout'),
				layers = context.data('atb-layout-layers');
			
			if ( !layoutData.hasPermission('edit') ) {
			
				alert("You do not have permission to edit this layout.");
				
			} else if ( layers && layers.parent && confirm("Remove all customisations from this layout?\nWARNING: This is action is irreversible.") ) {

				var freshLayout = $.overlay(layers.parent);

				// Retain id, name & description
				freshLayout.layoutId = layoutData.layoutId;
				freshLayout.name = layoutData.name;
				freshLayout.description = layoutData.description;

				// Set the parent id
				freshLayout.parentId = layoutData.parentId;

				// Clear CSS fields
				freshLayout.importCSS = "/* empty */";
				freshLayout.customCSS = "/* empty */";
				freshLayout.ieCSS = "/* empty */";

				// Save and reload the cleared layout
				ManageLayoutsDWR.setLayout(freshLayout, function() {
					$(event.target).trigger('atb-layout-reload');
				});
		    }
	    })
	    
	    // Delete the current layout
		.bind('atb-layout-delete', function(event) {
			var context = dataContext(event),
				layoutData = context.data('atb-layout'),
				parentId = layoutData.parentId,
				hierarchy = context.data('atb-hierarchy');
			
			if ( !layoutData.hasPermission('edit') ) {
			
				alert("You do not have permission to delete this layout.");
				
			} else if ( (hierarchy[layoutData.layoutId] || []).length ) {
			
				alert("You can't delete this layout because other layouts are based on it. Delete or move the child layouts first.");
				
			} else if ( confirm("Delete layout: " + layoutData.layoutId + ", Are you sure?") ) {
			
				ManageLayoutsDWR.deleteLayout(layoutData.layoutId, function(status) {
					if (status) {
						return alert(status);
					}
					
					$(event.target)
						// Select parent layout
						.trigger('atb-layout-select', [ parentId ])
					
						// Reload the layout hierarchy
						.trigger('atb-hierarchy-load');
				});
			}
		})
		
		// Create a new layout as a child of the current layout
		.bind('atb-layout-new', function(event) {
			cloneLayout(event, true);
		})
		
		// Duplicate the current layout as a sibling of itself
		.bind('atb-layout-duplicate', function(event) {
			cloneLayout(event, false);		
		});

})(jQuery);

