/*global jQuery, alert, confirm, document */

// Main Layout Manager Agent

// requires: layout-data.js, fields.js, revert.js

jQuery(function($) {

	$('#layout-manager')
		.bind('atb-layout-select', function(event, layoutId, opts) {
			if ( !(opts && opts.noConfirm) &&
					($(this).is('.modified') || !!$('.dialog-modified', this).length) ) {

				// TODO: Get message from markup
				if (!confirm("You have unsaved changes, continue and lose the changes?")) {
					// Trigger a cancel event to ensure that the tree reverts to the correct selection
					$(event.target).trigger('atb-layout-cancel');
					
					// Prevent propagation
					return false;
				}
			}
		})
		
		// Setup layout fields when layout is loaded/updated
		.bind('atb-layout-updated', function(event, layoutId, layoutData) {
			// Set the read only status
			$(this).toggleClass('readonly', !layoutData.hasPermission('edit'));

			// Set document title
			document.title = layoutData.layoutId + " - Adaptavist Theme Builder Layout";

			// Setup fields
			$('.layout-field', this).atbSetupField();
			$(this).trigger('atb-fields-ready');
		})
		
		// Setup layout fields when a new element is loaded
		.bind('atb-ready', function(event) {
			$('.layout-field', event.target).atbSetupField();
			$(event.target).trigger('atb-fields-ready');
		})
		
		// Update the modified status after a change
		.bind('atb-field-changed atb-layout-updated', function(event) {
			var layoutEdits = $.data(this, 'atb-layout'),
				layoutOriginal = layoutEdits.getPrototype();
			
			$(this).toggleClass('modified', !$.isEqual(layoutEdits, layoutOriginal) );
		})
		
		// Close dialogs
		.bind('dialog-close', function(event) {
			$(event.target).closest(':role(dialog)').attr('aria-hidden', true);
		});


	$('#layout-manager-main')
		.trigger('atb-hierarchy-load')
		.trigger('atb-palette-load')
		.trigger('atb-permissions-load')
		.roleSetup()
		.trigger('atb-layout-select', [ $('#layout-manager').attr('data-default-layout') || 'DEFAULT' ]);

});

