/*global jQuery, NlsTree, NlsTreeDD, nlsddSession */

// Layout Tree Agent

// presentation: #layout-manager-tree
// binds: atb-hierarchy-updated, atb-layout-updated
// triggers: atb-layout-select, atb-layout-reparent

jQuery(function($) {

	var context = $('#layout-manager'),

		iconPath = $('#confluence-context-path').attr('content') +
			"/download/resources/com.adaptavist.confluence.themes.sitebuilder:sitebuilder/icons/",
		iconStyles = {
			readonly: ["document_lock.png","document_lock.png"],
			layout: ["document.png","document.png"],
			unsaved: ["document_warning.png","document_warning.png"],
			newlayout: ["document_add.png","document_add.png"]
		};

    function getIcons(style) {
        return iconStyles[style] ? iconPath+iconStyles[style][0]+","+iconPath+iconStyles[style][1] : null;
    }

	function selectLayout( event, layoutId ) {
		$(event.target).trigger('atb-layout-select', [ layoutId ]);
	}

	function moveLayout( event ) {
		var parentId = nlsddSession.destData.xtra.id,
			dragged = nlsddSession.srcData;
		
		if ( parentId ) {
			$.each(dragged, function(i, node) {
				if ( node && node.xtra.id ) {
					$(event.target).trigger('atb-layout-reparent', [ node.xtra.id, parentId ]);
				}
			});
		}
	}

	function refreshTree( nlsTree, hierarchy ) {
		nlsTree.removeChilds('DEFAULT', false);
		
		(function nodes( parentId ) {
			$.each(hierarchy[parentId] || [], function(i, childId) {
				nlsTree.add(childId, parentId, childId, "", getIcons("layout"), true, false,
					{ id: childId, parent: parentId }, "");
				nlsTree.setDrag(childId, true, false);
				nlsTree.setDrop(childId, true, false);

				nodes(childId);
			});
		})('DEFAULT');
		
		nlsTree.reloadNode('DEFAULT', true);
		nlsTree.expandAll();
	}

	function initNlsTree() {
		var nlsTree = new NlsTree('layoutTree'),
			nlsTreeDD = new NlsTreeDD('layoutTree');

		nlsTree.treeOnClick = selectLayout;
		nlsTreeDD.onNodeDrop = moveLayout;
		
		// Add node for the DEFAULT layout
		nlsTree.add('DEFAULT', 0, 'DEFAULT', '', getIcons('readonly'), true, false,
			{ id: 'DEFAULT', parent: 'DEFAULT' }, "Default layout (read only)");
		nlsTree.setDrag('DEFAULT', false, false);
		nlsTree.setDrop('DEFAULT', true, false);
		
		// Create an element for the tree attributes and render them
		nlsTree.renderAttributes( $('<div/>').appendTo(context).idrefs() );
		
		// Render the tree
		nlsTree.render('layout-manager-tree');
		
		return nlsTree;
	}

	context
		.bind('atb-hierarchy-updated', function(event, data) {
			var tree = $('#layout-manager-tree'),
				nlsTree = tree.data('atb-nlstree'),
				layoutData = $(this).data('atb-layout');
			
			if ( !nlsTree ) {
				// Initiate the tree widget
				nlsTree = initNlsTree();
				tree.data('atb-nlstree', nlsTree);
			}
			
			refreshTree(nlsTree, data);
			
			if ( layoutData ) {
				nlsTree.selectNodeById(layoutData.layoutId);
			}
		})
		
		.bind('atb-layout-updated atb-layout-cancel', function(event) {
			var layoutData = $(this).data('atb-layout'),
				nlsTree = $('#layout-manager-tree').data('atb-nlstree');
			
			if ( nlsTree && layoutData ) {
				nlsTree.selectNodeById(layoutData.layoutId);
			}
		});

});

