/*jslint bitwise: true, eqeqeq: true, nomen: true, undef: true */
/*global jQuery, ThemeAdminDWR, dwr */

// requires: jquery, mvc, mvc-dwr, defer, tablesorter

jQuery(function($) {
	
	var panel = $('#managespaces-panel');
	
	function busy() { panel.busy(true); }
	function done() { panel.busy(false); }
	
	// TODO: Move this to somewhere reusable
	$.fn.zebra = function() {
		return this
			.removeClass('rowAlternate')
			.filter(':not(.hidden):odd')
			.addClass('rowAlternate')
			.end();
	};

	var saveFields = {
		'name': true,
		'themeId': true,
		'layoutId': true,
		'layoutLock': true
	};
	
	var setup = function() {
	
//		$.adaptavist.layouts.setup(panel);
	
		// Init model-view bindings
		$(panel).mvcInit();
		
		// Init tablesorter
		$('#managespaces-table', panel)
			.tablesorter({
				textExtraction: function(node) {
					var val = $('[ref]:first', node).mvc().val();
					return  (val !== undefined && val !== null) ? val.toString() : '';
				}
			})
			.addClass('tablesorter')
//			.addClass('ui-button-notext')
			.find('> tbody > tr')
			.zebra();
//			.find('> td > select[ref=layoutId]')
//			.each($.adaptavist.layouts.button);		// Add layout browser buttons

		// Save changes to Spaces data
		$.mvc('/managespaces/spaces').bind({
			batch: function(events) {
				if (events.length > 1) {
					busy();
					dwr.engine.beginBatch();
				}
			},
			each: function(event) {
				var that = $.mvc(event.target);
				var field = that.key();
				
				if (field in saveFields) {
					var value = that.val();
					var key = that.key('key').val();
			
					busy();
					ThemeAdminDWR.setValue(key, field, value, done);
				}
			},
			batchEnd: function(events) {
				if (events.length > 1) {
					dwr.engine.endBatch();
					done();
				}
			}
		});
		
		// Filtering
		$.mvc('/managespaces/filter').bind(function() {			
			var criteria = this.val();
			
			var tableRows = $('#managespaces-table > tbody > tr', panel);
			
			var keys = $.trim(criteria.key.words);
			var names = $.trim(criteria.name.words);
			var teams = $.trim(criteria.teams.words);
			
			if (keys === "" && names === "" && teams === "" &&
					criteria.type.global && criteria.type.personal) {

				// No filtering required
				$.mvc('/managespaces/spaces/*/hidden').del();
				tableRows.removeClass('hidden');
				// TODO: Set class via a substitution template on the class attr or binding

			} else {
			
				var filter = function(value) {
					if (!criteria.type.global && value.type !== 'personal') { return false; }
					if (!criteria.type.personal && value.type === 'personal') { return false; }
					
					var search = function(words, body, anyall) {
						words = words.toLowerCase().split(/\s+/);
						body = body.toLowerCase();
					
						if (words.length === 0) { return true; }
					
						var valid = (anyall === 'all');
					
						$.each(words, function(i,w) {
							var present = (body.indexOf(w) >= 0);
							if ((valid && !present) || (!valid && present)) {
								valid = !valid;
								return false;
							}
						});
						return valid;
					};
				
					if (!search(teams, value.teams.join(' '), criteria.teams.anyall)) { return false; }
					if (!search(names, value.name, criteria.name.anyall)) { return false; }
					if (!search(keys, value.key, criteria.key.anyall)) { return false; }
				
					return true;
				};

				$.mvc('/managespaces/spaces/*').walk({
					target: function(parent, key) {
						var show = filter(parent[key]);
						var that = $.mvc(this).addStep('-');
						if (show) {
							tableRows.eq(key).removeClass('hidden');
						} else {
							that.key('selected').val(false);
							tableRows.eq(key).addClass('hidden');
						}
						that.key('hidden').val(!show);
					}
				});
			}
			
			tableRows.zebra();
		});
		
		// Bulk Update
		$.mvc('/managespaces/bulkupdate/update').bind({
			type: 'set add',
			each: function() {
				// Get the update criteria
				var update = $.mvc('/managespaces/bulkupdate').val();
			
				// Filter out non-selected rows
				var selected = $.mvc('/managespaces/spaces/*')
						.filter(function(v) { return (v.selected === true); })
						.addStep('-');
			
				// Apply changes for each field
				$.each(['themeId', 'layoutId', 'layoutLock'], function(n, key) {
					if (update[key] !== '@nochange@') {
						selected.key(key).val(update[key]);
					}
				});
			
				// Reset the action status
				this.del();
			}
		});
		
		// Select All/None
		$.mvc('/managespaces/action/select').bind({
			type: 'set add',
			each: function() {
				// Should we select all or none?
				var select = (this.val() === 'all');
			
				// Set the selected prop of visible rows
				$.mvc('/managespaces/spaces/*')
					.filter(function(v) { return (v.hidden !== true); })
					.addStep('selected')
					.val(select);
			
				// Reset the action status
				this.del();
			}
		});

		// Trigger a change event on text inputs when the user pauses typing
		$(':text').keyup($.defer(300,function(){$(this).trigger('change');}));
		
		panel.trigger('ready');
	};
	
//	$.adaptavist.layouts.dwr();
	
	// Load data into the model
	$.mvc('/layouts/-')
//				.key('hierarchy').dwr(ManageLayoutsDWR.getHierarchy)
				.key('names').dwr(ThemeAdminDWR.getLayoutNames);

	$.mvc('/managespaces/-')
		.key('themes').dwr(ThemeAdminDWR.getThemes)
//		.key('layouts').dwr(ThemeAdminDWR.getLayoutNames)
		.key('spaces').dwr(ThemeAdminDWR.getSpaces)
		.dwrGo(setup);

});

