/*jslint bitwise: true, eqeqeq: true, nomen: true, undef: true */
/*global jQuery, ThemeAdminDWR, document */

// requires: jquery, mvc, mvc-dwr, autocomplete

jQuery(function($) {

	var panel = $('#permissions-panel');
	
	function busy() { panel.busy(true); }
	function done() { panel.busy(false); }
	
	function setup() {
		$.mvc('/ac/perms/[manageLayouts,editAdvanced,editCSS]').walk({
			target: function(parent, key) {
				// Set the default permissions
				if (!(key in parent)) {
					parent[key] = ['confluence-administrators'];
				}
				
				// Convert the array into an object, to ensure that groups are unique
				var map = {};
				$.each(parent[key], function(n,v) {
					map[v] = true;
				});
				parent[key] = map;
			},
			outgoing: $.mvc.walk.construct
		});
		
		$(panel).mvcInit();
		
		function aclNextItem(ev) {
			var next = $(ev.target).closest('li').next('li');
			if (next.length === 0) {
				next = $(ev.target).closest('div.acl').find('input');
			}
			return next;
		}
		
		function aclInitItem() {
			$(this)
				// Apply/remove style for a focused item
				.bind('focus', function() { $(this).addClass('ui-state-focus'); })
				.bind('blur', function() { $(this).removeClass('ui-state-focus'); });
			
			// Fix focusing in IE when clicking within the item
			if ($.browser.msie) {
				$(this)
					.bind('click', function() { this.focus(); });
			}
		}
		
		$('div.acl', panel)
			.bind('click', function(ev) {
				if (ev.target === this) {
					// Focus the input field when blank space is clicked with the list area
					$('input', this)[0].focus();
				}
			})
			.find('>ul')
				.bind('mvc-repeat-add', function(ev, item) {
					aclInitItem.apply(item);
				})
				.find('>li')
					.each(aclInitItem)
				.end()
				
				// Move to prev/next item if LEFT/RIGHT key pressed
				.bind('keydown', function(ev) {
					var focus;
					switch (ev.keyCode) {
					case $.ui.keyCode.LEFT:
					case $.ui.keyCode.UP:
					case $.ui.keyCode.BACKSPACE:
						focus = $(ev.target).closest('li').prev('li');
						break;
					case $.ui.keyCode.RIGHT:
					case $.ui.keyCode.DOWN:
					case $.ui.keyCode.DELETE:
						focus = aclNextItem(ev);
						break;
					case $.ui.keyCode.HOME:
					case $.ui.keyCode.PAGE_UP:
						focus = $(ev.target).closest('ul').find('>li:first');
						break;
					case $.ui.keyCode.END:
					case $.ui.keyCode.PAGE_DOWN:
						focus = $(ev.target).closest('div.acl').find('input');
						break;
					}
					switch (ev.keyCode) {
					case $.ui.keyCode.DELETE:
					case $.ui.keyCode.BACKSPACE:
						$(ev.target).closest('li').mvc('mvc-repeat').del();
						break;
					}
					if (focus && focus.length) {
						focus[0].focus();
						ev.stopPropagation();
						ev.preventDefault();
					}
				})
				
				// Delete the item if the button is clicked
				.bind('click', function(ev) {
					if ($(ev.target).is('button')) {
						$(ev.target).closest('li').mvc('mvc-repeat').del();
					}
				})
			.end()
			.find('input')
				.autocomplete({
					source: function(request, response) {
						var acl = $(this.element).closest('div.acl').mvc().val();
						// escape regex characters
						var matcher = new RegExp($.ui.autocomplete.escapeRegex(request.term), "i");
						return $.grep($.mvc('/ac/roles').val(), function(value) {
							// also filter out existing items
							return matcher.test(value) && !acl[value];
						});
					},
					delay: 0,
					change: function(ev, ui) {
						if (ui) {
							$.mvc('/ac/perms').addStep($.mvc(this).key(),'-').key(ui.item.result).val(true);
						}

						// Clear the text field
						$(this).val('');
					}
				})

				.bind('keydown', function(ev) {
					if (ev.keyCode === $.ui.keyCode.LEFT) {
						// If caret is at the far-left of the field, move focus to the last item
						var caret;
						if (this.selectionEnd !== undefined) {
							caret = this.selectionEnd;
						}
						if (caret === 0) {
							$(this).parent().find('>ul>li:last').each(function() { this.focus(); });
							ev.stopPropagation();
							ev.preventDefault();
						}
					}
				});
		
		$.mvc('/ac/perms/*').bind(function() {
			var key = this.key(),
				val = [];
			
			// Convert permissions back to an array
			$.each(this.val(), function(k,v) { val.push(k);	});
			
			// Commit the permissions
			busy();
			ThemeAdminDWR.setGlobalPermission(key, val, done);
		});
		
		panel.trigger('ready');
	}
	
	// Load global permissions and groups that may be assigned the permissions
	$.mvc('/ac/-')
		.key('perms').dwr(ThemeAdminDWR.getGlobalPermissions)
		.key('roles').dwr(ThemeAdminDWR.getGroups)
		.dwrGo(setup);
});

